/*
	SRG - Squid Report Generator
	Copyright 2003 University of Waikato

	This file is part of SRG.

	SRG is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	SRG is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SRG; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/
#include "srg.h"
#include "config.h"
#include "prototypes.h"

/* Compares two date strings of the format YYYYMMMDD-YYYYMMDD. Returns true if
 * date1 is less than date2.
 * Dates are compared on the start date first and then the end date if
 * necessary
 */
int compare_datename(const char *date1, const char *date2)
{
	
	int rv=0;
	char tmp1[10];
	char tmp2[10];

	/* Exit Immediately if we weren't passed valid dates */
	if(date1==NULL || date2==NULL)
		return -1;
	if (strlen(date1) != 19 || strlen(date2) != 19)
		return -1;

	/* Extract the start dates */
	strncpy(tmp1, date1, 9);
	strncpy(tmp2, date2, 9);
	tmp1[9] = tmp2[9] = '\0';
	
	rv = compare_datepart(tmp1, tmp2);
	
	/* If start dates were identical, compare end dates */
	if (rv==0) {
		strncpy(tmp1, date1+10, 9);
		strncpy(tmp2, date2+10, 9);
		tmp1[9] = tmp2[9] = '\0';
	
		rv = compare_datepart(tmp1, tmp2);
	}

	/* Convert greater than to false */
	if (rv==-1)
		rv=0;
	
	return rv;
	
}

/* Compares two date strings of the format YYYYMMMDD. Returns true if date1 is
 * less than date2 false if they are equal or -1 if date1 is greater than 
 * date2.
 */
int compare_datepart(const char *date1, const char *date2) {

	char year1[5], year2[5];
	char month1[4], month2[4];
	char day1[3], day2[3];
	
	/* Exit Immediately if we weren't passed valid dates */
	if(date1==NULL || date2==NULL)
		return -1;
	if (strlen(date1) != 9 || strlen(date2) != 9)
		return -1;
	
	/* Break the start dates out into their components */
	strncpy(year1, date1, 4);
	year1[4] = '\0';
	strncpy(month1, date1+4, 3);
	month1[3] = '\0';
	strncpy(day1, date1+7, 2);
	day1[2] = '\0';
	strncpy(year2, date2, 4);
	year2[4] = '\0';
	strncpy(month2, date2+4, 3);
	month2[3] = '\0';
	strncpy(day2, date2+7, 2);
	day2[2] = '\0';

	/* Compare Years */
	if (atoi(year1) < atoi(year2))
		return 1;
	else if (atoi(year1) > atoi(year2))
		return -1;
	
	/* Compare Months */
	if (getMonthByName(month1) < getMonthByName(month2))
		return 1;
	else if (getMonthByName(month1) > getMonthByName(month2))
		return -1;
	
	/* Compare Days */
	if (atoi(day1) < atoi(day2))
		return 1;
	else if (atoi(day1) > atoi(day2))
		return -1;
	
	/* Greater than or Equal */
	return 0;
}

/* Generates a suitable date string from the specified start and end times.
 * The generated string will be in the format YYYYMMMDD-YYYYMMDD and will be
 * asprintf'd in to the supplied char* 
 *
 * Returns 0 on success, > 0 on failure
 */
char *generate_datename(const time_t start, const time_t end) {

	if (start < 1 || end < 1)
		return NULL;

	char *outstr = NULL;
	char sday[3];
	char eday[3];
	sprintf(sday, "%02d", localtime(&start)->tm_mday);
	sprintf(eday, "%02d", localtime(&end)->tm_mday);
	asprintf(&outstr, "%d%s%s-%d%s%s", localtime(&start)->tm_year+1900, 
			month_names[localtime(&start)->tm_mon], sday,
			localtime(&end)->tm_year+1900, 
			month_names[localtime(&end)->tm_mon], eday);

	return outstr;	
}

/* Write the HTML header */
void html_header(FILE *outfile) {


	/* Header & Title */
	if (srg.usephp && srg.phpheader!=NULL) {
		/* Include the specified PHP header */
		fprintf(outfile, "<?php\ninclude_once('%s');\n?>\n", 
			srg.phpheader);
		if (srg.authenticate) {
			fprintf(outfile, "<?php if (can_view(\"-1\")) { "
				"?>\n\t");
		}
	} else {
		/* Output a static HTML Header */
		fprintf(outfile, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML "
				"4.01 Transitional//EN\" \"http://www.w3."
				"org/TR/html4/loose.dtd\">\n");
		fprintf(outfile, "<html>\n<head>\n<title>%s</title>\n", 
			srg.title);
		fprintf(outfile, "<meta name=\"generator\" value=\""
			"SRG %s (%s)\">\n", version, HOME_URL);
		fprintf(outfile, "<link rel=\"stylesheet\" href=\"%s\" type=\""
				"text/css\">\n</head>\n\n", srg.cssfile);
		/* Output some basic layout and headers */
		fprintf(outfile, "<body>\n<div align=\"center\"><h1>%s</h1>"
				"</div>\n\n", srg.title);
	}
	
}

/* Write the HTML footer */
void html_footer(FILE *outfile) {
	
	fprintf(outfile, "<div align=\"center\" class=\"srgfooter\"><a href=\""
		"%s\">Generated by SRG %s</a></div>\n\n", HOME_URL, version);

	if (srg.usephp && srg.phpfooter!=NULL) {
		/* Include the specified PHP footer */
		fprintf(outfile, "<?php\ninclude_once('%s');\n\n",
			srg.phpfooter);
	} else {
		fprintf(outfile, "\n</body>\n</html>");
	}

}


/* Determines whether the specified Cache Result Code is a cache hit */
bool is_cache_hit(const char * inputCode) {

	// Extract the result code from the status code
	bool hit=false;
	char *resultCode;
	char *tmp = strdup(inputCode);
	
	break_string(tmp, '/');
	resultCode = tmp;
	if (!resultCode) {
		fprintf(stderr, "Incorrect result code! %s\n", inputCode);
		exit(1);
	}
	
	if (srg.debug)
		fprintf(stderr, "Checking if %s is a cache hit\n", 
			inputCode);

	if (strcasecmp(resultCode, "TCP_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_REFRESH_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_REF_FAIL_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_IMS_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_NEGATIVE_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_MEM_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "TCP_OFFLINE_HIT")==0)
		hit = true;
	else if (strcasecmp(resultCode, "UDP_HIT")==0)
		hit = true;

	resultCode = NULL;
	free(tmp);

	return hit;

}

/* Determines whether the specified Cache Result Code was a deny */
bool is_cache_denied(const char * inputCode) {

	// Extract the result code from the status code
	bool denied=false;
	char *resultCode;
	char *tmp = strdup(inputCode);
	
	break_string(tmp, '/');
	resultCode = tmp;
	if (!resultCode) {
		fprintf(stderr, "Incorrect result code! %s\n", inputCode);
		exit(1);
	}
	
	if (srg.debug)
		fprintf(stderr, "Checking if %s is a cache denied\n", 
			inputCode);

	if (strcasecmp(resultCode, "TCP_DENIED")==0)
		denied = true;
	else if (strcasecmp(resultCode, "UDP_DENIED")==0)
		denied = true;

	resultCode = NULL;
	free(tmp);

	return denied;

}

/* Converts a short 3-char month abbreviation into it's numerical equivalent */
int getMonthByName(const char *month_name) {

	if (strcasecmp(month_name,"Jan")==0) 
		return 0;
	else if (strcasecmp(month_name,"Feb")==0) 
		return 1;
	else if (strcasecmp(month_name,"Mar")==0) 
		return 2;
	else if (strcasecmp(month_name,"Apr")==0) 
		return 3;
	else if (strcasecmp(month_name,"May")==0) 
		return 4;
	else if (strcasecmp(month_name,"Jun")==0) 
		return 5;
	else if (strcasecmp(month_name,"Jul")==0) 
		return 6;
	else if (strcasecmp(month_name,"Aug")==0) 
		return 7;
	else if (strcasecmp(month_name,"Sep")==0) 
		return 8;
	else if (strcasecmp(month_name,"Oct")==0) 
		return 9;
	else if (strcasecmp(month_name,"Nov")==0) 
		return 10;
	else if (strcasecmp(month_name,"Dec")==0) 
		return 11;	

	assert(0);	
	return -1;
}

